/*
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * com.ibm.rational.wvcm.stp.cc.CcTrustManagerCallback
 *
 * (C) Copyright IBM Corporation 2011, 2016.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */
package com.ibm.rational.wvcm.stp.cc;

import java.security.KeyStore;
import java.security.Principal;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.List;

/**
 * This callback is used to handle SSL Certificate problems when initiating a
 * connection to a remote CCRC WAN server over HTTPS. The callback is not
 * invoked if the certificate is stored and trusted on the client already.
 */
public interface CcTrustManagerCallback {
    
    /**
     * Provides the caller details about what is wrong with a certificate
     */
    public enum CertificateStatus {
        /**
         * The certificate is not trusted
         */
        CERTIFICATE_NOT_TRUSTED,
        
        /**
         * The certificate's date is out of range
         */
        CERTIFICATE_DATE_OUT_OF_RANGE,
        
        /**
         * The certificate's name does not match what is trusted in the store
         */
        CERTIFICATE_NAME_MISMATCH
    }
    
    /**
     * The caller provides a response in reply to the callback to indicate
     * how to handle the certificate problem.
     */
    public enum CertificateResponse {
        
        /**
         * Temporarily accept the certificate for this session.
         */
        ACCEPT_CERTIFICATE_TEMPORARILY,
        
        /**
         * Accept this certificate and install it into the key store
         */
        ACCEPT_AND_INSTALL_CERTIFICATE,
        
        /**
         * Reject this certificate for this session.
         */
        REJECT_CERTIFICATE
    }
    
    /**
     * Callback is invoked when there is a problem with the certificate provided
     * by the server. Check the <code>CertificateException</code> and the 
     * <code>CertificateStatus</code> for more details. Respond to the
     * certificate problem using a <code>CertificateResponse</code>
     * 
     * @param cert Java x.509 certificate
     * @param status List of certificate problems
     * @param certEx Java certificate exception
     * @return CertificateResponse accept/install/reject
     */
    CertificateResponse getCertificateResponse(
            X509Certificate cert,
            List<CertificateStatus> status,
            CertificateException certEx);
    
    /**
     * Choose a valid client certificate
     * 
     * @param keyStore The keyStore or null
     * @param keyType the key algorithm type name(s), ordered with the most-preferred key type first.
     * @param issuers the list of acceptable CA issuer subject names or null if it does not matter which issuers are used.
     * @return The alias of the selected client certificate
     */
    String chooseClientCertificate(
            KeyStore keyStore, 
            String[] keyType,
            Principal[] issuers);

}
